@@ -33,11 +33,12 @@ fn find_subset_fast_only_positive(arr: Vec<u32>, value: usize, max_length: usize
 /// * `targets` - An array.
 /// * `max_key_length` - An integer.
 /// * `max_target_length` - An integer.
+/// * `n_candidates` - An integer.
 #[pyfunction]
-#[pyo3(text_signature = "(keys, targets, max_key_length, max_target_length /)")]
-fn sequence_matcher(mut keys: Vec<i32>, mut targets: Vec<i32>, max_key_length: usize, max_target_length: usize) -> PyResult<Vec<Vec<(Vec<i32>, Vec<i32>)>>> {
+#[pyo3(text_signature = "(keys, targets, max_key_length, max_target_length, n_candidates /)")]
+fn sequence_matcher(mut keys: Vec<i32>, mut targets: Vec<i32>, max_key_length: usize, max_target_length: usize, n_candidates: usize) -> PyResult<Vec<Vec<(Vec<i32>, Vec<i32>)>>> {
     use crate::dp_module::*;
-    Ok(dp::sequence_matcher(&mut keys, &mut targets, max_key_length, max_target_length))
+    Ok(dp::sequence_matcher(&mut keys, &mut targets, max_key_length, max_target_length, n_candidates))
 }
 
 #[pymodule]
